use strict;

use DE_EPAGES::Core::API::Program::nodejs;
use DE_EPAGES::Core::API::SystemCommand qw( SystemCommand );
use DE_EPAGES::Core::API::File qw( CorrectPathSlashes );
use DE_EPAGES::Core::API::Script qw ( RunScript );

sub usage {
    print <<END_USAGE;
Description:
    Runs 'node epnode6.js {epnode6.js options}'

Usage:
    perl $0 {epnode6.js options}
END_USAGE
    exit 2;
}

sub Main {
    local $| = 1;

    usage() unless scalar @ARGV;

    my (%Parameters, $Option, $Help);
    my @HelpOptions = qw(-h -v --help --version);

    foreach (@ARGV) {
        if (/^-+/) {
            $Option = $_;
            $Parameters{$Option} = '';
            $Help = 1 if $Option ~~ @HelpOptions;
            next;
        }
        next if not $Option or $Parameters{$Option};
        $Parameters{$Option} = $_;
    }

    my $Node = DE_EPAGES::Core::API::Program::nodejs->new;
    my $EpNodeJS = "$ENV{EPAGES_PERL}/bin/nodejs.d/lib/node_modules/node-epages6/bin/epnode6.js";

    # run node command
    SystemCommand([$Node->command, CorrectPathSlashes($EpNodeJS), @ARGV]);
}

RunScript( 'Sub' => \&Main );
