#!/bin/bash

BN=${0##*/}
DN=$(cd $(dirname $0) ; pwd)
. /etc/default/epages6

KARMA="$DN/nodejs.d/lib/node_modules/karma/bin/karma"

ls -d $DN/nodejs.sh $KARMA >/dev/null 2>&1 && HAS_KARMA=1 || HAS_KARMA=

usage() {
  [[ -n $1 ]] && echo -e "Error:\n  $1"
  echo "Usage:"
  echo "  $0 [--conf <configFile>] {<options> | <command> [<options>]}"
  echo "Options:"
  echo "  configFile : path to karma-config.js"
  echo "     default : EPAGES_WEBROOT/StoreTypes/*/*/lib/testsuite/karma-config.js"
  echo "               if --conf '' then don't use configFile"
  echo "  command    : one of: start init run completion (as karma defines)"
  echo "  options    : options as karma defines"
  echo "Examples:"
  echo "  $BN --help"
  echo "  $BN --conf '' --help"
  echo "  $BN start --help"
  echo "  $BN --conf '' start"
  echo "  $BN --conf \$EPAGES_WEBROOT/StoreTypes/6.17.4/Store/lib/testsuite/karma-config.js start"
  echo "  $BN start \$EPAGES_WEBROOT/StoreTypes/6.17.4/Store/lib/testsuite/karma-config.js"
  echo "  $BN start"
  [[ -n $HAS_KARMA ]] && echo && $DN/nodejs.sh "$KARMA" --help
  exit 1
}

unset RUN KARMA_CONF

case $1 in
  --conf) KARMA_CONF=$2 ; RUN=$3 ; shift ; shift ; shift ;;
  --version) $DN/nodejs.sh "$KARMA" --version; exit $? ;;
  ''|-*) usage ;;
  *) RUN=$1 ; shift ;;
esac

[[ -z $RUN ]] && usage

if [[ -z ${!KARMA_CONF*} ]] ; then
  for STORETYPE in $(ls -dt "$EPAGES_CONFIG"/StoreTypes.d/*/*.conf) ; do
    VERSION=${STORETYPE%/*}
    VERSION=${VERSION##*/}
    STORETYPE=${STORETYPE##*/}
    STORETYPE=${STORETYPE%.conf}
    [[ -f "$EPAGES_WEBROOT"/StoreTypes/$VERSION/$STORETYPE/lib/testsuite/karma-config.js ]] \
    && break
  done
  KARMA_CONF="$EPAGES_WEBROOT"/StoreTypes/$VERSION/$STORETYPE/lib/testsuite/karma-config.js
fi

ls -d $DN/nodejs.sh $KARMA $KARMA_CONF >/dev/null || exit 1

echo  "INFO [$BN] NODEJS=$DN/nodejs.sh"
echo  "INFO [$BN] KARMA=$KARMA"
echo  "INFO [$BN] KARMA_CONF=$KARMA_CONF"
echo  "INFO [$BN] KARMA_COMMAND=$RUN"
echo  "INFO [$BN] KARMA_OPTIONS=$@"

$DN/nodejs.sh "$KARMA" $RUN "$KARMA_CONF" "$@"
