/*jslint node:true*/
module.exports = function (grunt) {
    'use strict';

    // Load all grunt tasks.
    require('load-grunt-tasks')(grunt);

    grunt.initConfig({
        jshint: {
            run: {
                options: {
                    jshintrc: true
                },
                src: ['./*.js', 'lib/**.js', 'bin/**.js', 'test/**.{js,json}']
            }
        },

        mochaTest: {
            run: {
                options: {
                    require: 'coverage/blanket',
                    reporter: 'spec'
                },
                src: ['test/*-test.js']
            },

            coverage: {
                options: {
                    reporter: 'html-cov',
                    quiet: true,
                    captureFile: 'coverage/coverage.html'
                },
                src: ['test/*-test.js']
            },

            // The travis-cov reporter will fail the tests if the
            // coverage falls below the threshold configured in package.json
            ci: {
                options: {
                    reporter: 'travis-cov'
                },
                src: ['test/*-test.js']
            }
        },

        watch: {
            test: {
                files: [
                    '*.js',
                    'lib/*.js',
                    'bin/*.js',
                    'coverage/*.js',
                    'test/*.js'
                ]
            }
        },

        clean: {
            'coverage': 'coverage/coverage.html'
        }

    });

    grunt.registerTask('test', function (file) {
        if (file) {
            grunt.config('jshint.run.src', 'test/' + file + '-test.js');
            grunt.config('mochaTest.run.src', 'test/' + file + '-test.js');
            grunt.config('mochaTest.coverage.src', 'test/' + file + '-test.js');
            grunt.config('mochaTest.ci.src', 'test/' + file + '-test.js');
        }

        grunt.task.run(['jshint:run', 'mochaTest']);
    });

    // grunt.event.on('watch', function (action, filepath) {
    //     grunt.task.run('test:' + filepath.split('.')[0]);
    // });

    grunt.registerTask('default', [
        'clean',
        'test'
    ]);
};