/*jslint indent:4,node:true*/
(function () {

    'use strict';

    var fs = require('fs.extra'),
        cp = require('child_process'),
        path = require('path'),
        async = require('async'),
        chownr = require('chownr');

    /**
     * Creates the documentation for the fiven files.
     *
     * @param {Object} options            The options
     * @param {String} options.input      A path to a JSON file which enumerates the files to
     *                                    include.
     * @param {String} options.cartridges The folder which contains the cartridges, e.g.
     *                                    /srv/epages/eproot/Cartridges
     * @param {String} options.output     The folder which will contain the output files. This
     *                                    folder will be deleted before the output files are
     *                                    written.
     * @param {Number} options.uid        The id of the user which will own the created files.
     * @param {Number} options.gid        The id of the group which will own the created files.
     */
    module.exports = function (options, callback) {
        var cmd,
            includes;

        cmd = 'jsduck ';

        async.waterfall([
            function (callback) {
                fs.mkdirp(options.output, function (error) {
                    callback(error);
                });
            },

            function (callback) {
                fs.readFile(options.input, callback);
            },

            function (json, callback) {
                includes = JSON.parse(json);
                includes.forEach(function (i) {
                    cmd += '"' + path.join(options.cartridges, i) + '" ';
                });

                cmd += ' -o "' + options.output + '"';

                cp.exec(cmd, {
                    maxBuffer: 1e10
                }, function (error) {
                    callback(error);
                });
            },

            function (callback) {
                if (options.uid && options.gid) {
                    chownr(options.output, options.uid, options.gid, callback);
                } else {
                    callback();
                }
            }
        ], callback);
    };

}());
