/*jslint node:true*/
(function () {
    'use strict';

    var minify = require('./minify'),
        fs = require('fs.extra'),
        path = require('path'),
        log = require('./log'),
        logger;

    log.add('File', {
        level: log.levels[log.getLogLevel(process.argv)],
        timestamp : function () {
            return new Date().getTime();
        },
        colorize: true,
        filename: path.join(process.env.EPAGES_LOG || "", 'epnode6-minify.log')
    });
    logger = log.getLogger({ module: 'lib/minify-worker.js' });

    process.on('message', function (m) {
        try {
            fs.mkdirp(path.dirname(m.target), function (e) {
                logger.debug('worker (id: ' + m.id + '), directory created: ' + path.dirname(m.target));

                minify(m.base, m.source, m.target, {'id': m.id}, function (err) {
                    logger.debug('worker process (id: ' + m.id + ') finished, m.target', m.target + 'err:' + err);

                    process.send(null);
                });
            });
        } catch (e) {
            e.source = m.source;
            process.send(e);
        }
    });
}());