/*jslint node:true, nomen:true*/
(function () {

    'use strict';

    var _ = require('underscore'),
        exec = require('child_process').exec,

        nodeSlashes = function (path) {
            return path.replace(/\\/g, '/');
        },

        perlJSAPI = nodeSlashes(process.env.EPAGES_PERL + '/bin/perl ' + __dirname + '/../child_process/jsapi.pl'),

        perlExec = function (args, callback, hook) {
            var params = [];

            _.each(args, function (value, name) {
                params.push('-' + name + ' "' + value.replace(/"/g, '\\"') + '"');
            });

            exec(perlJSAPI + ' ' + params.join(' '), function (error, stdout, stderr) {
                var data;

                if (!error) {
                    try {
                        data = JSON.parse(stdout);
                    } catch (e) {
                        error = e;
                    }
                }

                if (!data && !error) {
                    error = stderr || 'Cannot get ' + params.join(', ') + ' from perlJS API.';
                }

                if (data && hook) {
                    data = hook(data);
                }

                if (error) {
                    callback(error.toString(), null);
                } else {
                    callback(null, data);
                }
            });
        },

        getters = {
            'cartridges': function (storeType, callback) {
                storeType = storeType || 'Store';

                perlExec({
                    sub: 'GetStoreTypeCartridges',
                    param: storeType
                }, callback, function (data) {

                    if (data) {
                        _.each(data, function (value, i) {
                            data[i] = nodeSlashes(value.replace(/::/g, '/'));
                        });
                    }

                    return data;
                });
            },

            'env': function (vars, callback) {
                var data = {};

                vars = vars || ['version'];

                _.each(vars, function (value) {
                    var env = process.env['EPAGES_' + value.toUpperCase()];

                    data[value.toLowerCase()] = typeof env === 'string' ? nodeSlashes(env) : env;
                });

                if (data.hasOwnProperty('version')) {
                    perlExec({
                        sub: 'GetVersion'
                    }, callback, function (version) {
                        data.version = version;

                        return data;
                    });
                } else {
                    callback(null, data);
                }
            },

            'uid': function (user, callback) {
                /*jslint sub:true*/
                exec('id -u ' + process.env['EPAGES_' + user.toUpperCase()], function (err, stdout) {
                    callback(err ? err.toString() : +stdout);
                });
            },

            'gid': function (group, callback) {
                /*jslint sub:true*/
                callback(+((process.env['EPAGES_' + group.toUpperCase()]).split(':')[1]));
            }
        },

        get = function (getter, options, callback) {
            if (typeof options === 'function') {
                callback = options;
                options = null;
            }

            if (getters[getter]) {
                getters[getter](options, callback);
            } else {
                callback('Unknown getter ' + getter, null);
            }
        };

    module.exports.get = get;
    exports.get = get;

}());
