/*jslint node:true, nomen:true*/

(function () {
    'use strict';

    var perl = require('./perl'),
        path = require('path'),
        async = require('async'),
        grunt = require('grunt');

    module.exports = function (cartridgesToCheck, callback) {

        var missingTestcases = [],
            numberOfModules = 0,
            result;

        async.waterfall([
            // prepare cartridges
            function (callback) {
                async.series({
                    path: function (callback) {
                        perl.get('env', ['cartridges'], callback);
                    },
                    cartridges: function (callback) {
                        if (!cartridgesToCheck) {
                            perl.get('cartridges', callback);
                        } else {
                            callback(null, cartridgesToCheck.split(' '));
                        }
                    }
                }, function (e, result) {
                    callback(e, result.path.cartridges, result.cartridges);
                });
            },

            // do the rest
            function (cartridgePath, cartridges, callback) {
                cartridges.forEach(function (cartridge) {

                    var jsPath = path.join(cartridgePath, cartridge, 'Data/Public/lib/'),
                        modules;

                    if (!grunt.file.isDir(jsPath)) {
                        result = 'Cartridge ' + cartridge + ' contains no JavaScript!';
                    } else {
                        modules = grunt.file.expand([
                            jsPath + 'ep/**/*.js',
                            jsPath + cartridge.toLowerCase() + '/**/*.js',
                            jsPath + 'ckeditor/plugins/ep*/*.js',
                            '!' + jsPath + 'ep/i18n/*',
                            '!' + jsPath + '**/*-test.js',
                            '!' + jsPath + '**/*-model.js',
                            '!' + jsPath + '**/*-collection.js'
                        ]);

                        numberOfModules += modules.length;

                        modules.forEach(function (module) {
                            if (!grunt.file.isFile(module.replace(/\.js$/, '-test.js'))) {
                                missingTestcases.push(module.replace(jsPath, ''));
                            }
                        });
                    }
                });

                callback();
            }
        ], function (e) {
            var numberOfmissingTestcases = String(missingTestcases.length),
                ratio,
                color;

            if (e) { callback(e); }

            if (numberOfmissingTestcases > 0) {
                // we have some missing testcases
                ratio = (numberOfModules - numberOfmissingTestcases) / numberOfModules;

                if (ratio > 0.75 && ratio !== 1) {
                    color = 'yellow';
                } else if (ratio > 0.5 && ratio < 0.75) {
                    color = 'magenta';
                } else {
                    color = 'red';
                }

                result = '\n';

                async.eachSeries(missingTestcases, function (testcase, callback) {
                    // ...still waiting...
                    result += '\t[X] '.red + 'Missing test case for ' + testcase.cyan + '\n';
                    callback();
                }, function () {
                    result += '\n==> We have ' + numberOfmissingTestcases[color] + ' missing test cases | ' +
                        numberOfModules + ' modules checked | ' +
                        String(ratio * 100).substr(0, 5)[color] + ' percent coverage.';

                    // ...dary!
                    callback(result);
                });
            } else {
                callback(result);
            }
        });
    };
}());