/*jslint indent:4,node:true,nomen:true*/
/*global describe, it, after*/
(function () {
    'use strict';

    var main = require('../main'),
        expect = require('expect.js'),
        sinon = require('sinon');

    describe('Checking main.js calling', function () {

        describe('module packLib', function () {

            it('should return exec handler for packLib', function () {
                expect(main.packLib).to.be.a(Function);
            });

            it('should return an error when called with wrong parameters', function () {
                main.packLib({}, function (error) {
                    expect(error).to.be('Parameter store_type is wrong or missing\n');
                });
            });

            it('should call actual module when parameters are correct', function (callback) {
                var stub = sinon.stub(main, 'packLib').callsArg(1),
                    args = {store_type: 'Store'};

                main.packLib(args, function () {
                    expect(stub.getCall(0).args[0]).to.be(args);
                    callback();
                });
            });

            after(function () {
                main.packLib.restore();
            });
        });

        describe('module minifyLib', function () {
            it('should return exec handler for minifyLib', function () {
                expect(main.minifyLib).to.be.a(Function);
            });

            it('should return an error when called with wrong parameters', function () {
                main.minifyLib({}, function (error) {
                    expect(error).to.be('Parameter store_type is wrong or missing\n');
                });
            });

            it('should call actual module when parameters are correct', function (callback) {
                var stub = sinon.stub(main, 'minifyLib').callsArg(1),
                    args = {store_type: 'Store'};

                main.minifyLib(args, function () {
                    expect(stub.getCall(0).args[0]).to.be(args);
                    callback();
                });
            });

            after(function () {
                main.minifyLib.restore();
            });
        });

        describe('module minify', function () {

            it('should return exec handler for minify', function () {
                expect(main.minify).to.be.a(Function);
            });

            it('should return an error when called with wrong parameters', function () {
                main.minify({}, function (error) {
                    expect(error).to.be('Parameter source is wrong or missing\nParameter target is wrong or missing\n');
                });
            });

            it('should call actual module when parameters are correct', function (callback) {
                var stub = sinon.stub(main, 'minify').callsArg(1),
                    args = {
                        source: './lib',
                        target: './lib-min'
                    };

                main.minify(args, function () {
                    expect(stub.getCall(0).args[0]).to.be(args);
                    callback();
                });
            });

            after(function () {
                main.minify.restore();
            });
        });

        describe('module buildUi', function () {

            it('should return exec handler for build', function () {
                expect(main.buildUi).to.be.a(Function);
            });

            it('should return an error when called with wrong parameters', function () {
                main.buildUi({}, function (error) {
                    expect(error).to.be('Parameter store_type is wrong or missing\n');
                });
            });

            it('should call actual module when parameters are correct', function (callback) {
                var stub = sinon.stub(main, 'buildUi').callsArg(1),
                    args = { store_type: 'Store' };

                main.buildUi(args, function () {
                    expect(stub.getCall(0).args[0]).to.be(args);
                    callback();
                });
            });

            after(function () {
                main.buildUi.restore();
            });
        });

        describe('module testcases', function () {

            it('should return exec handler for testcases', function () {
                expect(main.testcases).to.be.a(Function);
            });

            it('should call actual module when parameters are correct', function (callback) {
                var stub = sinon.stub(main, 'testcases').callsArg(1);

                main.testcases({ cartridge: 'DE_EPAGES/JQuery' }, function () {
                    expect(stub.getCall(0).args[0]).to.eql({ cartridge: 'DE_EPAGES/JQuery' });
                    callback();
                });
            });

            after(function () {
                main.testcases.restore();
            });
        });

        describe('module builddoc', function () {

            it('should return exec handler for builddoc', function () {
                expect(main.minify).to.be.a(Function);
            });

            it('should return an error when called with wrong parameters', function () {
                main.builddoc({}, function (error) {
                    expect(error).to.be('Parameter include is wrong or missing\n');
                });
            });

            it('should call actual module when parameters are correct', function (callback) {
                var stub = sinon.stub(main, 'builddoc').callsArg(1),
                    args = { inlcude: '.' };

                main.builddoc(args, function () {
                    expect(stub.getCall(0).args[0]).to.be(args);
                    callback();
                });
            });

            after(function () {
                main.builddoc.restore();
            });
        });
    });
}());