/*jslint node:true*/
/*global describe,before,it,after*/

(function () {

    'use strict';

    var expect = require('expect.js'),
        check  = require('../lib/testcases'),
        perl = require('../lib/perl'),
        sinon = require('sinon'),
        path = require('path'),
        stub;

    describe('Running testcases.js', function () {

        before(function () {
            stub = sinon.stub(perl, 'get');

            stub.withArgs('cartridges').yields(null, [
                'PARENT/CartridgeA',
                'PARENT/CartridgeB',
                'PARENT/CartridgeD',
                'PARENT/CartridgeE'
            ]);

            stub.withArgs('env', ['cartridges']).yields(null, {cartridges: path.join(__dirname, 'fs/cartridges')});
        });

        it('should check for existence of testcases and fail (single cartridge provided)', function (callback) {
            check('PARENT/CartridgeA', function (error) {
                // check relevant parts of the error message (because different handling of special characters)
                var testError = /Missing test case for/.test(error) && /parent\/cartridgea/i.test(error);

                expect(testError).to.be(true);

                // 1 missing test case, 2 modules checked.
                callback();
            });
        });

        it('should fail to find provided cartridge', function (callback) {
            check('This_does_not_exist', function (error) {
                expect(error).to.be('Cartridge This_does_not_exist contains no JavaScript!');
                callback();
            });
        });

        it('should check for existence of testcases and pass (single cartridge provided)', function (callback) {
            check('PARENT/CartridgeB', function (error) {
                expect(error).to.be(undefined);
                callback();
            });
        });

        it('should check for existence of testcases (all cartridges)', function (callback) {
            check(undefined, function (error) {
                expect(/parent\/cartridgea\/myModuleWithoutTestcase.js/.test(error)).to.be(true);
                // 2 missing test cases, 5 modules checked.
                callback();
            });
        });

        after(function () {
            stub.restore();
        });

    });

}());
